﻿using Syncfusion.Windows.Forms.PdfViewer;
using System;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Mvc;
using Syncfusion.JavaScript.DataVisualization.Models;
using Syncfusion.JavaScript.DataVisualization.Models.Diagram;
using Syncfusion.JavaScript.DataVisualization.DiagramEnums;
using System.Text.RegularExpressions;
using Syncfusion.JavaScript.DataVisualization.Models.Collections;
using Syncfusion.JavaScript;
using Microsoft.AspNetCore.Hosting;

namespace Uploadsettings.Controllers.Diagram
{
    public partial class FileExplorerController : Controller
    {
        public FileExplorerOperations operation;
        // GET: /<controller>/
        public ActionResult FileExplorerFeatures()
        {
            return View();
        }
        public FileExplorerController(IHostingEnvironment hostingEnvironment)
        {
            this.operation = new FileExplorerOperations(hostingEnvironment.ContentRootPath);
        }
        public ActionResult Download(FileExplorerParams args)
        {
            return operation.Download(args.Path, args.Names);
        }
        public ActionResult Upload(FileExplorerParams args)
        {
            operation.Upload(args.FileUpload, args.Path);
            return Json("");
        }
        public ActionResult GetImage(FileExplorerParams args)
        {
            return operation.GetImage(args.Path);
        }
        public ActionResult GetPDFImage(string path)
        {
            string fullPath = path;
            PdfiumViewer.FPDF_AddRef();
            Stream input = System.IO.File.OpenRead(fullPath);
            int id = StreamManager.Register(input);
            IntPtr doc = PdfiumViewer.FPDF_LoadCustomDocument(input, null, id);
            PdfiumViewer.FPDF_GetDocPermissions(doc);
            PdfiumNative.PdfRenderFlags flags = PdfiumNative.PdfRenderFlags.CorrectFromDpi;
            int pageCount = PdfiumViewer.FPDF_GetPageCount(doc);
            double width;
            double height;
            int pagesizes = PdfiumViewer.FPDF_GetPageSizeByIndex(doc, 0, out width, out height);
            int Width = (int)width;
            int Height = (int)height;
            int rotation = 0;
            IntPtr pages = PdfiumViewer.FPDF_LoadPage(doc, 0);
            Bitmap bitmapImage = new Bitmap(Width, Height);
            BitmapData data = bitmapImage.LockBits(new System.Drawing.Rectangle(0, 0, Width, Height), ImageLockMode.ReadWrite, bitmapImage.PixelFormat);
            IntPtr createdpages = PdfiumViewer.FPDFBitmap_CreateEx(Width, Height, 4, data.Scan0, bitmapImage.Width * 4);
            uint background = (flags & PdfiumNative.PdfRenderFlags.Transparent) == 0 ? 0xFFFFFFFF : 0x00FFFFFF;
            PdfiumViewer.FPDFBitmap_FillRect(createdpages, 0, 0, Width, Height, background);
            PdfiumViewer.FPDF_RenderPageBitmap(createdpages, pages, 0, 0, Width, Height, rotation, 0);
            PdfiumViewer.FPDFBitmap_Destroy(createdpages);
            bitmapImage.UnlockBits(data);
            MemoryStream memoryStream = new MemoryStream();
            bitmapImage.Save(memoryStream, ImageFormat.Png);
            memoryStream.Seek(0, SeekOrigin.Begin);
            return File(memoryStream, "image/jpeg");
        }
        public ActionResult FileActionDefault([FromBody] FileExplorerParams args)
        {
            switch (args.ActionType)
            {
                case "Read":
                    return Json(operation.Read(args.Path, args.ExtensionsAllow));
                case "CreateFolder":
                    return Json(operation.CreateFolder(args.Path, args.Name));
                case "Paste":
                    return Json(operation.Paste(args.LocationFrom, args.LocationTo, args.Names, args.Action, args.CommonFiles));
                case "Remove":
                    return Json(operation.Remove(args.Names, args.Path, args.SelectedItems));
                case "Rename":
                    return Json(operation.Rename(args.Path, args.Name, args.NewName, args.CommonFiles));
                case "GetDetails":
                    return Json(operation.GetDetails(args.Path, args.Names));
                case "Search":
                    return Json(operation.Search(args.Path, args.ExtensionsAllow, args.SearchString, args.CaseSensitive));
            }
            return Json("");
        }
    }
}
