﻿using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using System.Text;
using System.IO;
using System.Reflection;
using Syncfusion.Windows.Forms.PdfViewer;

namespace Syncfusion.Windows.Forms.PdfViewer
{
    internal class PdfiumViewer
    {
        [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
        public delegate int FPDF_GetBlockDelegate(IntPtr param, uint position, IntPtr buffer, uint size);

        public static readonly FPDF_GetBlockDelegate _getBlockDelegate = FPDF_GetBlock;
        private static readonly string LockString =("e362349b-001d-4cb2-bf55-a71606a3e36f");
        public static void FPDF_AddRef()
        {
            lock (LockString)
            {
                PdfiumNative.FPDF_AddRef();
            }
        }

        public static void FPDF_Release()
        {
            lock (LockString)
            {
                PdfiumNative.FPDF_Release();
            }
        }
        public static IntPtr FPDF_LoadDocument(string path, string password)
        {
            lock (LockString)
            {
                return PdfiumNative.FPDF_LoadDocument(path, password);
            }
        }
        public static int FPDF_GetPageCount(IntPtr document)
        {
            lock (LockString)
            {
                return PdfiumNative.FPDF_GetPageCount(document);
            }
        }
        public static int FPDF_GetPageSizeByIndex(IntPtr document, int page_index, out double width, out double height)
        {
            lock (LockString)
            {
                return PdfiumNative.FPDF_GetPageSizeByIndex(document, page_index, out width, out height);
            }
        }
        public static IntPtr FPDF_LoadPage(IntPtr document, int page_index)
        {
            lock (LockString)
            {
                return PdfiumNative.FPDF_LoadPage(document, page_index);
            }
        }
        public static IntPtr FPDFBitmap_CreateEx(int width, int height, int format, IntPtr first_scan, int stride)
        {
            lock (LockString)
            {
                return PdfiumNative.FPDFBitmap_CreateEx(width, height, format, first_scan, stride);
            }
        }
        public static IntPtr FPDFBitmap_Create(int width, int height, int alpha)
        {
            lock (LockString)
            {
                return PdfiumNative.FPDFBitmap_Create(width, height, alpha);
            }
        }
        public static void FPDFBitmap_FillRect(IntPtr bitmapHandle, int left, int top, int width, int height, uint color)
        {
            lock (LockString)
            {
                PdfiumNative.FPDFBitmap_FillRect(bitmapHandle, left, top, width, height, color);
            }
        }
        public static void FPDF_RenderPageBitmap(IntPtr bitmapHandle, IntPtr page, int start_x, int start_y, int size_x, int size_y, int rotate, PdfiumNative.FPDF flags)
        {
            lock (LockString)
            {
                PdfiumNative.FPDF_RenderPageBitmap(bitmapHandle, page, start_x, start_y, size_x, size_y, rotate, flags);
            }
        }
        public static IntPtr FPDFBitmap_Destroy(IntPtr bitmapHandle)
        {
            lock (LockString)
            {
                return PdfiumNative.FPDFBitmap_Destroy(bitmapHandle);
            }
        }
        public static IntPtr FPDF_LoadMemDocument(byte[] data_buf, int size, string password)
        {
            lock (LockString)
            {
                return PdfiumNative.FPDF_LoadMemDocument(data_buf, size, password);
            }
        }
        public static IntPtr FPDFBitmap_GetBuffer(IntPtr bitmap)
        {
            lock (LockString)
            {
                return PdfiumNative.FPDFBitmap_GetBuffer(bitmap);
            }
        }
        public static void FPDF_CloseDocument(IntPtr bitmapHandle)
        {
            lock (LockString)
            {
                PdfiumNative.FPDF_CloseDocument(bitmapHandle);
            }
        }
        public static int FPDFBitmap_GetStride(IntPtr bitmap)
        {
            lock (LockString)
            {
                return PdfiumNative.FPDFBitmap_GetStride(bitmap);
            }
        }
        public static int FPDF_GetPageHeight(IntPtr page)
        {
            lock (LockString)
            {
                return PdfiumNative.FPDF_GetPageHeight(page);
            }
        }
        public static int FPDF_GetPageWidth(IntPtr page)
        {
            lock (LockString)
            {
                return PdfiumNative.FPDF_GetPageWidth(page);
            }
        }
        public static int FPDFPage_GetRotation(IntPtr page)
        {
            lock (LockString)
            {
                return PdfiumNative.FPDFPage_GetRotation(page);
            }
        }
        public static uint FPDF_GetDocPermissions(IntPtr document)
        {
            lock (LockString)
            {
                return PdfiumNative.FPDF_GetDocPermissions(document);
            }
        }
        public static IntPtr FPDF_LoadCustomDocument(Stream input, string password, int id)
        {
            Syncfusion.Windows.Forms.PdfViewer.PdfiumNative.FPDF_FILEACCESS access = new Syncfusion.Windows.Forms.PdfViewer.PdfiumNative.FPDF_FILEACCESS();
            access.m_FileLen = (uint)input.Length;
            access.m_GetBlock = Marshal.GetFunctionPointerForDelegate(_getBlockDelegate);
            access.m_Param = (IntPtr)id;
            lock (LockString)
            {
                return PdfiumNative.FPDF_LoadCustomDocument(access, password);
            }
        }

        private static int FPDF_GetBlock(IntPtr param, uint position, IntPtr buffer, uint size)
        {
            Stream stream = StreamManager.Get((int)param);
            if (stream == null)
                return 0;
            byte[] managedBuffer = new byte[size];

            stream.Position = position;
            int read = stream.Read(managedBuffer, 0, (int)size);
            if (read != size)
                return 0;

            Marshal.Copy(managedBuffer, 0, buffer, (int)size);
            return 1;
        }
    }
    internal class PdfiumNative
    {
        static PdfiumNative()
        {
            string paths = "D:/Incident Works/FileExplroer/ASP.NET Core/Uploadsettings";
            if (paths != null)
            {
                paths = Path.Combine(paths, "Pdfium");
                if (Directory.Exists(paths))
                {
                    paths = Path.Combine(paths, IntPtr.Size == 4 ? "x86" : "x64");
                    paths = Path.Combine(paths, "Pdfium.dll");
                    LoadLibrary(paths);
                }
            }
        }
        [DllImport("kernel32", SetLastError = true, CharSet = CharSet.Unicode)]
        private static extern IntPtr LoadLibrary([MarshalAs(UnmanagedType.LPTStr)] string lpFileName);
        [DllImport("Pdfium.dll")]
        public static extern int FPDFBitmap_GetStride(IntPtr bitmap);
        [DllImport("Pdfium.dll")]
        public static extern void FPDF_AddRef();
        [DllImport("Pdfium.dll")]
        public static extern uint FPDF_GetDocPermissions(IntPtr document);
        [DllImport("Pdfium.dll")]
        public static extern void FPDF_Release();

        [DllImport("pdfium.dll", CharSet = CharSet.Ansi)]
        public static extern IntPtr FPDF_LoadCustomDocument([MarshalAs(UnmanagedType.LPStruct)]FPDF_FILEACCESS access, string password);

        [DllImport("Pdfium.dll", CharSet = CharSet.Ansi)]
        public static extern IntPtr FPDF_LoadMemDocument(SafeHandle data_buf, int size, string password);

        [DllImport("Pdfium.dll", CharSet = CharSet.Ansi)]
        public static extern IntPtr FPDF_LoadMemDocument(byte[] data_buf, int size, string password);
        [DllImport("Pdfium.dll", CharSet = CharSet.Ansi)]
        public static extern IntPtr FPDF_LoadDocument(string path, string password);
        [DllImport("Pdfium.dll", CharSet = CharSet.Ansi)]
        public static extern FPDF_ERR FPDF_GetLastError();

        [DllImport("Pdfium.dll")]
        public static extern int FPDFText_GetText(IntPtr page, int start_index, int count, byte[] result);
        [DllImport("Pdfium.dll")]
        public static extern IntPtr FPDF_LoadPage(IntPtr document, int page_index);
        [DllImport("Pdfium.dll")]
        public static extern uint FPDFImageObj_GetImageDataRaw(IntPtr document, byte[] buffer, uint length);
        [DllImport("Pdfium.dll")]
        public static extern IntPtr FPDFText_LoadPage(IntPtr page);

        [DllImport("Pdfium.dll")]
        public static extern IntPtr FPDFPage_GetObject(IntPtr page, int index);

        [DllImport("Pdfium.dll")]
        public static extern IntPtr FPDFBitmap_CreateEx(int width, int height, int format, IntPtr first_scan, int stride);

        [DllImport("Pdfium.dll")]
        public static extern void FPDF_RenderPage(IntPtr dc, IntPtr page, int start_x, int start_y, int size_x, int size_y, int rotate, FPDF flags);

        [DllImport("Pdfium.dll")]
        public static extern void FPDF_RenderPageBitmap(IntPtr bitmapHandle, IntPtr page, int start_x, int start_y, int size_x, int size_y, int rotate, FPDF flags);

        [DllImport("Pdfium.dll")]
        public static extern IntPtr FPDFBitmap_Create(int width, int height, int alpha);

        [DllImport("Pdfium.dll")]
        public static extern void FPDFBitmap_FillRect(IntPtr bitmapHandle, int left, int top, int width, int height, uint color);

        [DllImport("Pdfium.dll")]
        public static extern IntPtr FPDFBitmap_Destroy(IntPtr bitmapHandle);
        [DllImport("Pdfium.dll")]
        public static extern int FPDF_GetPageWidth(IntPtr page);

        [DllImport("Pdfium.dll")]
        public static extern int FPDF_GetPageHeight(IntPtr page);
        [DllImport("Pdfium.dll")]
        public static extern int FPDF_GetPageCount(IntPtr document);
        [DllImport("Pdfium.dll")]
        public static extern int FPDF_GetPageSizeByIndex(IntPtr document, int page_index, out double width, out double height);
        [DllImport("Pdfium.dll")]
        public static extern IntPtr FPDFBitmap_GetBuffer(IntPtr bitmap);

        [DllImport("Pdfium.dll")]
        public static extern void FPDF_CloseDocument(IntPtr page);

        [DllImport("Pdfium.dll")]
        public static extern int FPDFPage_GetRotation(IntPtr page);
        public enum FPDF_ERR : uint
        {
            FPDF_ERR_SUCCESS = 0,		// No error.
            FPDF_ERR_UNKNOWN = 1,		// Unknown error.
            FPDF_ERR_FILE = 2,		// File not found or could not be opened.
            FPDF_ERR_FORMAT = 3,		// File not in PDF format or corrupted.
            FPDF_ERR_PASSWORD = 4,		// Password required or incorrect password.
            FPDF_ERR_SECURITY = 5,		// Unsupported security scheme.
            FPDF_ERR_PAGE = 6		// Page not found or content error.
        }
        [Flags]
        public enum FPDF
        {
            ANNOT = 0x01,
            LCD_TEXT = 0x02,
            NO_NATIVETEXT = 0x04,
            GRAYSCALE = 0x08,
            DEBUG_INFO = 0x80,
            NO_CATCH = 0x100,
            RENDER_LIMITEDIMAGECACHE = 0x200,
            RENDER_FORCEHALFTONE = 0x400,
            PRINTING = 0x800,
            REVERSE_BYTE_ORDER = 0x10
        }
        [StructLayout(LayoutKind.Sequential)]
        public class FPDF_FILEACCESS
        {
            public uint m_FileLen;
            public IntPtr m_GetBlock;
            public IntPtr m_Param;
        }
        public enum PdfRenderFlags
        {
            /// <summary>
            /// No flags.
            /// </summary>
            None = 0,
            /// <summary>
            /// Render for printing.
            /// </summary>
            ForPrinting = FPDF.PRINTING,
            /// <summary>
            /// Set if annotations are to be rendered.
            /// </summary>
            Annotations = FPDF.ANNOT,
            /// <summary>
            /// Set if using text rendering optimized for LCD display.
            /// </summary>
            LcdText = FPDF.LCD_TEXT,
            /// <summary>
            /// Don't use the native text output available on some platforms.
            /// </summary>
            NoNativeText = FPDF.NO_NATIVETEXT,
            /// <summary>
            /// Grayscale output.
            /// </summary>
            Grayscale = FPDF.GRAYSCALE,
            /// <summary>
            /// Limit image cache size.
            /// </summary>
            LimitImageCacheSize = FPDF.RENDER_LIMITEDIMAGECACHE,
            /// <summary>
            /// Always use halftone for image stretching.
            /// </summary>
            ForceHalftone = FPDF.RENDER_FORCEHALFTONE,
            /// <summary>
            /// Render with a transparent background.
            /// </summary>
            Transparent = 0x1000,
            /// <summary>
            /// Correct height/width for DPI.
            /// </summary>
            CorrectFromDpi = 0x2000
        }
    }
}
